package analyzer

import "io"
import "fmt"
import "testing"
import "strings"
import "git.tebibyte.media/sashakoshka/fspl/errors"
import "git.tebibyte.media/sashakoshka/fspl/parser"

func testStringErr (
	test *testing.T,
	errMessage  string,
	errLine     int,
	errColumn   int,
	input       string,
) {
	testReaderErr (
		test,
		errMessage, errLine, errColumn,
		strings.NewReader(input))
}

func testReaderErr (
	test *testing.T,
	errMessage  string,
	errRow      int,
	errStart    int,
	inputs ...io.Reader,
) {
	ast := parser.Tree { }
	for index, stream := range inputs {
		err := ast.Parse(fmt.Sprintf("stream%d.fspl", index), stream)
		if err != nil {
			test.Error("parser returned error:", err)
			return
		}
	}

	tree := Tree { }
	err := tree.Analyze(ast)
	if err == nil {
		test.Error("analyzer did not return error")
		return
	}
	
	got := err.(errors.Error)
	gotMessage := got.Error()
	gotRow     := got.Position().Row   + 1
	gotStart   := got.Position().Start + 1
	correct :=
		gotMessage == errMessage &&
		gotRow     == errRow     &&
		gotStart   == errStart
	if !correct {
		test.Log("errors do not match")
		test.Log("got:\n" + errors.Format(got))
		test.Logf("correct:\n%v:%v: %v", errRow, errStart, errMessage)
		test.Fail()
	}
}

func testString (test *testing.T, input string) {
	testReader(test, strings.NewReader(input))
}

func testReader (test *testing.T, inputs ...io.Reader) {
	ast := parser.Tree { }
	for index, stream := range inputs {
		err := ast.Parse(fmt.Sprintf("stream%d.fspl", index), stream)
		if err != nil {
			test.Error("parser returned error:", err)
			return
		}
	}

	tree := Tree { }
	err := tree.Analyze(ast)
	if err != nil {
		test.Error("analyzer returned error:", err)
		return
	}
}
