package parser

import "git.tebibyte.media/sashakoshka/fspl/lexer"
import "git.tebibyte.media/sashakoshka/fspl/errors"
import "git.tebibyte.media/sashakoshka/fspl/entity"

func (this *Parser) parseLiteralInt () (*entity.LiteralInt, error) {
	err := this.expect(lexer.Int)
	if err != nil { return nil, err }
	defer this.next()
	
	value, err := parseInteger(this.value())
	if err != nil { return nil, errors.Errorf(this.pos(), err.Error()) }
	
	return &entity.LiteralInt {
		Position: this.pos(),
		Value:    int(value), // TODO struct should store as int64
	}, nil
}

func (this *Parser) parseLiteralFloat () (*entity.LiteralFloat, error) {
	err := this.expect(lexer.Float)
	if err != nil { return nil, err }
	defer this.next()
	
	value, err := parseFloat(this.value())
	if err != nil { return nil, errors.Errorf(this.pos(), err.Error()) }
	
	return &entity.LiteralFloat {
		Position: this.pos(),
		Value:    value,
	}, nil
}

func (this *Parser) parseLiteralString () (*entity.LiteralString, error) {
	err := this.expect(lexer.String)
	if err != nil { return nil, err }
	defer this.next()
	
	return &entity.LiteralString {
		Position:  this.pos(),
		ValueUTF8: this.value(),
	}, nil
}

func (this *Parser) parseLiteralBoolean () (*entity.LiteralBoolean, error) {
	err := this.expectValueDesc("boolean", lexer.Ident, "true", "false")
	if err != nil { return nil, err }
	defer this.next()
	
	return &entity.LiteralBoolean {
		Position: this.pos(),
		Value:    this.value() == "true",
	}, nil
}

func (this *Parser) parseLiteralNil () (*entity.LiteralNil, error) {
	err := this.expectValueDesc("Nil", lexer.Ident, "nil")
	if err != nil { return nil, err }
	defer this.next()
	
	return &entity.LiteralNil {
		Position: this.pos(),
	}, nil
}

func (this *Parser) parseLiteralArrayCore (pos errors.Position) (*entity.LiteralArray, error) {
	err := this.expect(lexer.Star)
	if err != nil { return nil, err }
	literal := &entity.LiteralArray {
		Position: pos,
	}
	this.next()

	for {
		err := this.expectDesc (
			"array literal element or end",
			appendCopy(startTokensExpression, lexer.RParen)...)
		if err != nil { return nil, err }
		if this.kind() == lexer.RParen { break }

		element, err := this.parseExpression()
		if err != nil { return nil, err }
		literal.Elements = append(literal.Elements, element)
	}
	literal.Position = literal.Position.Union(this.pos())
	this.next()
	
	return literal, nil
}

func (this *Parser) parseLiteralStructCore (pos errors.Position) (*entity.LiteralStruct, error) {
	err := this.expect(lexer.Dot)
	if err != nil { return nil, err }
	literal := &entity.LiteralStruct {
		Position: pos,
	}
	this.next()

	for {
		err := this.expectDesc (
			"struct literal member or end",
			appendCopy(startTokensExpression, lexer.RParen)...)
		if err != nil { return nil, err }
		if this.kind() == lexer.RParen { break }

		member, err := this.parseMember()
		if err != nil { return nil, err }
		literal.Members = append(literal.Members, member)
	}
	literal.Position = literal.Position.Union(this.pos())
	this.next()
	
	return literal, nil
}
