package parser

import "strconv"
import "git.tebibyte.media/sashakoshka/fspl/lexer"
import "git.tebibyte.media/sashakoshka/fspl/entity"

var descriptionSignature = "signature"
var startTokensSignature = []lexer.TokenKind { lexer.LBracket }

func (this *Parser) parseSignature () (*entity.Signature, error) {
	err := this.expectDesc(descriptionSignature, startTokensSignature...)
	if err != nil { return nil, err }
	pos := this.pos()
	err = this.expectNext(lexer.Ident)
	if err != nil { return nil, err }
	signature := &entity.Signature{
		Position: pos,
		Name:     this.value(),
	}
	this.next()

	for {
		err := this.expectDesc (
			"parameter or signature end",
			appendCopy(startTokensDeclaration, lexer.RBracket)...)
		if err != nil { return nil, err }
		if this.kind() == lexer.RBracket { break }

		parameter, err := this.parseDeclaration()
		if err != nil { return nil, err }
		signature.Arguments = append(signature.Arguments, parameter)
	}
	signature.Position = signature.Position.Union(this.pos())
	this.next()

	if this.kind() == lexer.Colon {
		this.next()
		ret, err := this.parseType()
		if err != nil { return nil, err }
		signature.Return = ret
	}
	
	return signature, nil
}

func (this *Parser) parseMember () (*entity.Member, error) {
	err := this.expectDesc("struct member", lexer.Ident)
	if err != nil { return nil, err }
	name := this.value()
	pos  := this.pos()

	err = this.expectNext(lexer.Colon)
	if err != nil { return nil, err }
	this.next()

	value, err := this.parseExpression()
	if err != nil { return nil, err }

	return &entity.Member {
		Position: pos,
		Name:     name,
		Value:    value,
	}, nil
}

func parseInteger (value string) (int64, error) {
	base := 10
	if len(value) > 0 && value[0] == '0' {
		base = 8
		if len(value) > 1 {
			switch value[1] {
			case 'x':
				value = value[2:]
				base = 16
			case 'b':
				value = value[2:]
				base = 1
			}
		}
	}
	
	integer, err := strconv.ParseInt(value, base, 64)
	if err != nil { return 0, err }
	return integer, nil
}

func parseFloat (value string) (float64, error) {
	float, err := strconv.ParseFloat(value, 64)
	if err != nil { return 0, err }
	return float, nil
}
