package parser

import "io"
import "os"
import "fmt"
import "git.tebibyte.media/sashakoshka/fspl/entity"

// Tree represents a parsed abstract syntax tree. It has no constructor and its
// zero value can be used safely.
type Tree struct {
	Declarations []entity.TopLevel
}

// String returns a string representation of the tree. The result of this will
// be syntactically valid.
func (this *Tree) String () string {
	out := ""
	for index, declaration := range this.Declarations {
		if index > 0 { out += "\n" }
		out += fmt.Sprint(declaration)
	}
	return out
}

// ParseFile parses the contents of the given file into the tree.
func (this *Tree) ParseFile (name string) error {
	file, err := os.Open(name)
	if err != nil { return err }
	defer file.Close()
	return this.Parse(name, file)
}

// Parse parses the contents of the given io.Reader into the tree.
func (this *Tree) Parse (name string, file io.Reader) error {
	parser, err := NewParser(name, file)
	if err != nil { return err }
	return parser.ParseInto(this)
}

// AddDeclaration adds a top-level entity to the tree.
func (this *Tree) AddDeclaration (topLevel ...entity.TopLevel) {
	this.Declarations = append(this.Declarations, topLevel...)
}
